#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "activate.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "update.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *configure_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo camera;
extern dc1394_feature_set feature_set;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];
extern char* trigger_mode_list[4];
extern char* channel_num_list[16];
extern int max_memory_channel;
extern int current_memory_channel;

void ActivateCapabilities()
{
  // ----------- Commander window --------------------------------------------
  
  // basic capabilities:
  ActivatePowerFrame();

  // trigger capabilities:
  ActivateTriggerFrame();

  // format capabilities:
  BuildFormatMenu();

  // update FPS menu according to format:
  UpdateFpsMenu();

  // memory capabilities
  ActivateMemoryFrame();

  // Activate ranges (min, max, current, capabilities)
  ActivateRange(porthole_window, FEATURE_FOCUS);
  ActivateRange(porthole_window, FEATURE_PAN);
  ActivateRange(porthole_window, FEATURE_TILT);
  ActivateRange(porthole_window, FEATURE_ZOOM);

  ActivateRange(aperture_window, FEATURE_EXPOSURE);
  ActivateRange(aperture_window, FEATURE_IRIS);
  ActivateRange(aperture_window, FEATURE_SHUTTER);
  ActivateRange(aperture_window, FEATURE_GAIN);
  ActivateRange(aperture_window, FEATURE_OPTICAL_FILTER);

  ActivateRange(color_window, FEATURE_BRIGHTNESS);
  ActivateRange(color_window, FEATURE_GAMMA);
  ActivateRange(color_window, FEATURE_SATURATION);
  ActivateRange(color_window, FEATURE_HUE);
  ActivateRange(color_window, FEATURE_WHITE_BALANCE);
  ActivateRange(color_window, FEATURE_SHARPNESS);

  ActivateRange(capture_window, FEATURE_CAPTURE_SIZE);
  ActivateRange(capture_window, FEATURE_CAPTURE_QUALITY);

  ActivateRange(temperature_window, FEATURE_TEMPERATURE);
  
}


void BuildFormatMenu()
{
  int err;
  quadlet_t modes, formats;

  // formats capabilities:
  err= dc1394_query_supported_formats(camera.handle, camera.id, &formats);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_0"),formats & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_1"),formats & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_2"),formats & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_6"),formats & (0x1<<25) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_7"),formats & (0x1<<24) );

  // corresponding modes:
  err= dc1394_query_supported_modes(camera.handle, camera.id, 0, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode5"),modes & (0x1<<26) );
  
  err= dc1394_query_supported_modes(camera.handle, camera.id, 1, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode5"),modes & (0x1<<26) );
  
  err= dc1394_query_supported_modes(camera.handle, camera.id, 2, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode5"),modes & (0x1<<26) );

  err= dc1394_query_supported_modes(camera.handle, camera.id, 6, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f6_mode0"),modes & (0x1<<31) );

  err= dc1394_query_supported_modes(camera.handle, camera.id, 7, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode5"),modes & (0x1<<26) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode6"),modes & (0x1<<25) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode7"),modes & (0x1<<24) );

}

void ActivateRange(GtkWidget* current_window, int feature)
{
  GtkAdjustment *adjustment, *adjustment2;

  if (feature_set.feature[feature].available)
    {
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_frame_list[feature]),TRUE);
      // one push auto:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_op_list[feature]),
			       feature_set.feature[feature].one_push);
      // on/off switching:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_power_list[feature]),
			       feature_set.feature[feature].on_off_capable);
      // manual mode:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_man_list[feature]),
			       feature_set.feature[feature].manual_capable);
      // auto mode:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_auto_list[feature]),
			       feature_set.feature[feature].auto_capable);

      // values:
      adjustment=(GtkAdjustment*)gtk_adjustment_new(feature_set.feature[feature].min,
						    feature_set.feature[feature].min,
						    feature_set.feature[feature].max,1,10,0);
      switch(feature)
	{
	  case FEATURE_WHITE_BALANCE:
	    adjustment2=(GtkAdjustment*)gtk_adjustment_new(feature_set.feature[feature].min,
							   feature_set.feature[feature].min,
							   feature_set.feature[feature].max,1,10,0);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "whitebal_BU_scale"),adjustment);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "whitebal_RV_scale"),adjustment2);break;
	  case FEATURE_TEMPERATURE:
	    adjustment2=(GtkAdjustment*)gtk_adjustment_new(feature_set.feature[feature].min,
							   feature_set.feature[feature].min,
							   feature_set.feature[feature].max,1,10,0);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "temp_goal_scale"),adjustment);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "temp_current_scale"),adjustment2);break;
	  default:
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, feature_scale_list[feature]),adjustment);
	}
    }
  UpdateRange(current_window,feature);
}

void
BuildTriggerModeMenu()
{
  int err, i, modes;
  quadlet_t value;
  GtkWidget* trigger_mode;
  GtkWidget* trigger_mode_menu;
  GtkWidget* glade_menuitem;

  err=dc1394_query_feature_characteristics(camera.handle,camera.id,FEATURE_TRIGGER,&value);

  modes=( (value & (0xF << 12))>>12 );
  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"trigger_mode"))); // remove previous menu

  trigger_mode = gtk_option_menu_new ();
  gtk_widget_ref (trigger_mode);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "trigger_mode", trigger_mode,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (trigger_mode);
  gtk_box_pack_start (GTK_BOX (lookup_widget(commander_window,"vbox25")), trigger_mode, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (trigger_mode), 1);

  trigger_mode_menu = gtk_menu_new ();

  if (modes) // at least one mode present
    { // external trigger available:
      for (i=0;i<=3;i++)
	{
	  if (modes & (0x1<<(3-i)))
	    {
	      glade_menuitem = gtk_menu_item_new_with_label (_(trigger_mode_list[i]));
	      gtk_widget_show (glade_menuitem);
	      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
	      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
				  GTK_SIGNAL_FUNC (on_trigger_mode_activate),
				  (int*)i);
	    }
	}
    }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (trigger_mode), trigger_mode_menu);
}


void
BuildMemoryChannelMenu()
{
  int err, i;
  quadlet_t value;
  GtkWidget* channel_num;
  GtkWidget* channel_num_menu;
  GtkWidget* glade_menuitem;

  current_memory_channel=0; // should be moved to update function (?)

  err=dc1394_query_basic_functionality(camera.handle,camera.id,&value);
  max_memory_channel=( value & (0xF) );

  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"memory_channel"))); // remove previous menu

  channel_num = gtk_option_menu_new ();
  gtk_widget_ref (channel_num);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "memory_channel", channel_num,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (channel_num);
  gtk_box_pack_start (GTK_BOX (lookup_widget(commander_window,"vbox33")), channel_num, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (channel_num), 1);

  channel_num_menu = gtk_menu_new ();

  for (i=0;i<=max_memory_channel;i++)
    {
      glade_menuitem = gtk_menu_item_new_with_label (_(channel_num_list[i]));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (channel_num_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_memory_channel_activate),
			  (int*)i); // i is an int passed in a pointer variable. This is 'normal'.
    }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (channel_num), channel_num_menu);

}

void
ActivateTriggerFrame()
{
  // always set the trigger frame on (because it contains the fps menu):
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_frame"),TRUE);

  // check external trigger feature presence:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_external"),
			   feature_set.feature[FEATURE_TRIGGER].available);

  // polarity change available:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_polarity"),
			   (feature_set.feature[FEATURE_TRIGGER].available) &&
			   (feature_set.feature[FEATURE_TRIGGER].polarity_capable));

  // build external trigger modes menu:  
  BuildTriggerModeMenu();
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_mode"),
			   (feature_set.feature[FEATURE_TRIGGER].available) &&
			   (feature_set.feature[FEATURE_TRIGGER].is_on));

}


void ActivatePowerFrame()
{
  quadlet_t basic_funcs;
  int err;
  // these two functions are always present:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_frame"),TRUE);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_reset"),TRUE);

  // activate if camera capable of power on/off:
  err=dc1394_query_basic_functionality(camera.handle,camera.id,&basic_funcs);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_on"),(basic_funcs & 0x1<<16));
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_off"),(basic_funcs & 0x1<<16));
}


void ActivateMemoryFrame()
{
  // load always present, so we can activate it:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"memory_frame"),TRUE);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"load_mem"),TRUE);

  // save not activated by default (it is not avail. for factory defaults channel):
  gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(commander_window,"save_mem")), FALSE);

  // activate the mem channel menu:
  BuildMemoryChannelMenu();
  gtk_widget_set_sensitive(lookup_widget(commander_window,"memory_channel"),TRUE);
}

